@@ -369,11 +369,19 @@ public:
       return;
     }
     
+    SGGeod previousPos = previous->pos;
+    // use correct location for runway, otherwise use threshold
+    // and get weird courses
+    if (previous->wpt->type() == "runway") {
+        FGRunway* rwy = static_cast<RunwayWaypt*>(previous->wpt.get())->runway();
+        previousPos = rwy->end();
+    }
+
     if (posValid) {
-      legCourseTrue = SGGeodesy::courseDeg(previous->pos, pos);
+      legCourseTrue = SGGeodesy::courseDeg(previousPos, pos);
       legCourseValid = true;
     } else if (isCourseConstrained()) {
-      double magVar = magVarFor(previous->pos);
+      double magVar = magVarFor(previousPos);
       legCourseTrue = wpt->headingRadialDeg() + magVar;
       legCourseValid = true;
     } // of pos not valid
@@ -464,8 +472,14 @@ public:
           turnExitAngle += theta;
         
           // move by the distance to compensate
-          double d = turnRadius * 2.0 * sin(theta * SG_DEGREES_TO_RADIANS);
-          turnExitPos = SGGeodesy::direct(turnExitPos, next.legCourseTrue, d);
+          double p = copysign(90, turnExitAngle);
+          const double offsetAngle = legCourseTrue + turnExitAngle - p;
+          SGGeod tc2 = SGGeodesy::direct(turnExitCenter,
+                                          offsetAngle,
+                                          turnRadius * 2.0);
+          
+
+          turnExitPos = SGGeodesy::direct(tc2, next.legCourseTrue + p , turnRadius);
           overflightCompensationAngle = -theta;
 
             // sign of angles will differ, so compute distances seperately
@@ -576,23 +590,28 @@ public:
 
   SGGeod pointAlongExitPath(double distanceM) const
   {
-      double theta = (distanceM / turnRadius) * SG_RADIANS_TO_DEGREES;
-      double p = copysign(90, turnExitAngle);
+      double basicTurnDistance = pathDistanceForTurnAngle(turnExitAngle);
+      
+      if (distanceM > basicTurnDistance) {
+          assert(flyOver);
+          assert(overflightCompensationAngle != 0.0);
+          double p = copysign(90, turnExitAngle);
 
-      if (flyOver && (overflightCompensationAngle != 0.0)) {
-          // figure out if we're in the compensation section
-          if (theta > turnExitAngle) {
-              // compute the compensation turn center - twice the turn radius
-              // from turnCenter
-              SGGeod tc2 = SGGeodesy::direct(turnExitCenter,
-                                             legCourseTrue - overflightCompensationAngle - p,
-                                             turnRadius * 2.0);
-              theta = copysign(theta - turnExitAngle, overflightCompensationAngle);
-              return SGGeodesy::direct(tc2,
-                                       legCourseTrue - overflightCompensationAngle + theta + p, turnRadius);
-          }
+          double distanceAlongCompensationTurn = distanceM - basicTurnDistance;
+          double theta = (distanceAlongCompensationTurn / turnRadius) * SG_RADIANS_TO_DEGREES;
+
+          // compute the compensation turn center - twice the turn radius
+          // from turnCenter
+          SGGeod tc2 = SGGeodesy::direct(turnExitCenter,
+                                         legCourseTrue + turnExitAngle - p,
+                                         turnRadius * 2.0);
+
+          theta = copysign(theta, overflightCompensationAngle);
+          return SGGeodesy::direct(tc2,
+                                   legCourseTrue + turnExitAngle + theta + p, turnRadius);
       }
-
+      
+      double theta = (distanceM / turnRadius) * SG_RADIANS_TO_DEGREES;
       theta = copysign(theta, turnExitAngle);
       double inboundCourse = legCourseTrue + (flyOver ? 0.0 : turnExitAngle);
       return pointOnExitTurnFromHeading(inboundCourse + theta);
@@ -1188,7 +1207,16 @@ double RoutePath::distanceForIndex(int index) const
 
 double RoutePath::distanceBetweenIndices(int from, int to) const
 {
-  return d->distanceBetweenIndices(from, to);
+    if (from < 0) {
+        from = 0;
+    }
+
+    const auto sz = static_cast<int>(d->waypoints.size());
+    if (to >= sz) {
+        to = sz - 1;
+    }
+
+    return d->distanceBetweenIndices(from, to);
 }
 
 SGGeod RoutePath::positionForDistanceFrom(int index, double distanceM) const
